package com.covas.Resources;

import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.UUID;

import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.transaction.Transactional;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.CookieParam;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.covas.Classes.Hash;
import com.covas.Entity.UsersEntity;
import com.covas.Json.UserSingle;

import org.eclipse.microprofile.jwt.Claims;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.jboss.logging.Logger;
import org.postgresql.shaded.com.ongres.scram.common.bouncycastle.base64.Base64;

@Produces(MediaType.APPLICATION_JSON)
@Path("users")
public class UsersRessources {
    private static final Logger LOGGER = Logger.getLogger(UsersRessources.class);
    @Inject
    JsonWebToken jwt;
 
    ///Function
    private Boolean checkUserCookie(String userCookie, UsersEntity users){
        String name = new String(Base64.decode(userCookie), StandardCharsets.UTF_8);
        if(!name.equals(users.pseudo) && (!users.status)){
            return false;
        }
        return true;
    }

    /// Appel HTTP

    ///GET
    @GET
    @RolesAllowed("Admin")
    public Response getUsers(){
        return Response.ok(UsersEntity.listAll()).build();
    }

    @GET
    @RolesAllowed("Admin")
    @Path("{id}")
    public Response getSingleUser(@PathParam("id") String id, @CookieParam("user") String userCookie, @Context SecurityContext ctx){
        if(!ctx.getUserPrincipal().getName().equals(jwt.getName())){
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).build();
        }
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        if(!checkUserCookie(userCookie, user)){
            return Response.status(Response.Status.FORBIDDEN).build();
        }
        UUID uid = UUID.fromString(id);
        UsersEntity users = UsersEntity.findById(uid);
        if(users == null){
            return Response.status(Response.Status.NOT_FOUND).build();
        }
        return Response.ok(users).build();
    }

    @GET
    @RolesAllowed("User")
    @Path("info")
    public Response getInfoUser(@Context SecurityContext ctx,  @CookieParam("user") String userCookie){
        if(!ctx.getUserPrincipal().getName().equals(jwt.getName())){
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).build();
        }
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        if (user == null){
            return Response.status(Response.Status.NOT_FOUND).build();
        }
        if(!checkUserCookie(userCookie, user)){
            return Response.status(Response.Status.FORBIDDEN).build();
        }
        return Response.ok(new UserSingle(user.name, user.pseudo, user.firstName)).build();
    }

    //PUT

    @PUT
    @RolesAllowed("Admin")
    @Consumes(MediaType.APPLICATION_JSON)
    @Transactional
    public Response createUser(@Context SecurityContext ctx, @CookieParam("user") String userCookie, UsersEntity users){
        if(!ctx.getUserPrincipal().getName().equals(jwt.getName())){
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).build();
        }
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        if(!checkUserCookie(userCookie, user)){
            return Response.status(Response.Status.FORBIDDEN).build();
        }

        UsersEntity usersOrig = UsersEntity.findByPseudo(users.pseudo);
        if(usersOrig != null){
            return Response.status(Response.Status.CONFLICT).build();
        }
        UsersEntity usersNew = new UsersEntity();
        usersNew.id = UUID.randomUUID();
        usersNew.name = users.name;
        usersNew.pseudo = users.pseudo;
        usersNew.firstName = users.firstName;
        usersNew.birth = LocalDate.of(users.birth.getYear(), users.birth.getMonth(), users.birth.getDayOfMonth());
        usersNew.created_at = LocalDateTime.now();
        usersNew.updated_at = LocalDateTime.now();
        usersNew.password = Hash.encryptSHA512(Base64.toBase64String(users.password.getBytes(StandardCharsets.UTF_8)));
        usersNew.roles = users.roles;
        usersNew.status = false;
        usersNew.persist();
        if(usersNew.isPersistent()){
            return Response.status(Response.Status.CREATED).build();
        }
        return Response.status(Response.Status.NO_CONTENT).build();
    }


}
